(

	local theCameras = #()
	
	local wbPresets = 
	#(
		#("Daylight",6500),
		#("Sunlight",5200),
		#("Shade",7000),
		#("Overcast",6000),
		#("Incandescent",3200),
		#("Fluorescent",4000),
		
		#("CIE A",2856),
		#("CIE D50",5003),
		#("CIE D55",5503),
		#("CIE D65",6504),
		#("CIE D75",7504),
		
		#("CIE F1",6430),
		#("CIE F2",4230),
		#("CIE F3",3450),
		#("CIE F4",2940),
		#("CIE F5",6350),
		#("CIE F6",4150),
		#("CIE F7",6500),
		#("CIE F8",5000),
		#("CIE F9",4150),
		#("CIE F10",5000),
		#("CIE F11",4000),
		#("CIE F12",3000),
		
		#("Halogen Warm",2800),
		#("Halogen",3200),
		#("Halogen Cool",4000),
		
		#("HID Ceramic Warm",3000),
		#("HID Ceramic Cool",4200),
		#("HID Quartz Warm",3200),
		#("HID Quartz Cool",4000),
		#("HID Mercury",3900),
		#("HID Phospor Mercury",4000),
		#("HID Xenon",6000),
		#("HID HP Sodium",2100),
		#("HID LP Sodium",1800)
	)

	fn copyCameraControllers inputCam =
	(
		if inputCam != undefined do
		(
			outputCam = vrayPhysicalCamera name:(uniquename(inputCam.name + "_V-Ray"))
			--most properties work this way
			for p in getpropnames inputCam do 
			(
				if isProperty outputCam p do --the property exists
				(
-- 					print ("found property " + p as string)
					if isPropertyAnimatable outputCam p and ((getPropertyController inputCam p) != undefined) then 
					(
-- 					print (p as string + " is animatable")
						setPropertyController outputCam p (getPropertyController inputCam p)
					)
					else
					(
-- 						print (p as string + " is NOT animatable")
						setProperty outputCam p (getProperty inputCam p)
					)
				)		
			)
			--a few do not.
			(
				case inputCam.shutter_unit_type of 
				(
					0: 
					(
						outputCam.type = 0
					)
					1:
					(
						outputCam.type = 2
					)
					2:
					(
						outputCam.type = 1
					)
					3:
					(
						--no correspondence
					)
				)
				--film
				try 
				(
					setPropertyController outputCam #film_width (getPropertyController inputCam #film_width_mm)
				)
				catch 
				(
					setProperty outputCam #film_width (getProperty inputCam #film_width_mm)
				)
				--focal
				try 
				(
					setPropertyController outputCam #focal_length (getPropertyController inputCam #focal_length_mm)
				)
				catch 
				(
					setProperty outputCam #focal_length (getProperty inputCam #focal_length_mm)
				)
				--moblur
				try 
				(
					setPropertyController outputCam #use_moblur (getPropertyController inputCam #motion_blur_enabled)
				)
				catch 
				(
					setProperty outputCam #use_moblur (getProperty inputCam #motion_blur_enabled)
				)	
				--exposure control
				setProperty outputCam #exposure ((getProperty inputCam #exposure_gain_type)+1)
				if classof SceneExposureControl.exposureControl != Physical_Camera_Exposure_Control or not SceneExposureControl.exposureControl.active do
				(
					setProperty outputCam #exposure 0 --if no physcamexpcontrol is active, turn exposure off
				)
				--tilting
				try 
				(
					setPropertyController outputCam #lens_horTilt (getPropertyController inputCam #horizontal_tilt_correction)
				)
				catch 
				(
					setProperty outputCam #lens_horTilt (getProperty inputCam #horizontal_tilt_correction)
				)			
				try 
				(
					setPropertyController outputCam #lens_Tilt (getPropertyController inputCam #vertical_tilt_correction)
				)
				catch 
				(
					setProperty outputCam #lens_Tilt (getProperty inputCam #vertical_tilt_correction)
				)			
				setProperty outputCam #lens_tilt_auto (getProperty inputCam #auto_vertical_tilt_correction)
				--vignetting
				setProperty outputCam #vignetting (getProperty inputCam #vignetting_enabled)			

				--shutter
				try 
				(
					setPropertyController outputCam #shutter_speed (getPropertyController inputCam #shutter_length_frames)
				)
				catch 
				(
					setProperty outputCam #shutter_speed (getProperty inputCam #shutter_length_frames)
				)
	--			No angle for shutter. always calculated for the max physical			
	-- 			try 
	-- 			(
	-- 				setPropertyController outputCam #shutter_angle (getPropertyController inputCam #vertical_tilt_correction)
	-- 			)
	-- 			catch 
	-- 			(
	-- 				setProperty outputCam #shutter_angle (getProperty inputCam #vertical_tilt_correction)
	-- 			)				
	-- 			try 
	-- 			(
	-- 				setPropertyController outputCam #shutter_offset (getPropertyController inputCam #vertical_tilt_correction)
	-- 			)
	-- 			catch 
	-- 			(
	-- 				setProperty outputCam #shutter_offset (getProperty inputCam #vertical_tilt_correction)
	-- 			)			
				try 
				(
					setPropertyController outputCam #latency (getPropertyController inputCam #shutter_length_seconds)
				)
				catch 
				(
					setProperty outputCam #latency (getProperty inputCam #shutter_length_seconds)
				)		

-- 				local colorCorr = 1.4388/1.4380
				local colorCorr = 1.0
				--White Balance
				case inputCam.white_balance_type of
				(
					0:	--preset-based
					(
						outputCam.whiteBalance_preset = 7 --temperature
						outputCam.temperature = wbPresets[inputCam.white_balance_illuminant+1][2] * colorCorr -- +1 to match index, second entry in the retrieved sub-array
					)
					1:	--temp based
					(
						outputCam.whiteBalance_preset = 7 --temperature
						try 
						(
							setPropertyController outputCam #temperature (getPropertyController inputCam #white_balance_kelvin)
						)
						catch 
						(
							setProperty outputCam #temperature ((getProperty inputCam #white_balance_kelvin)*colorCorr)
						)
					)
					2:	--Custom
					(
						outputCam.whiteBalance_preset = 0 --custom
						try 
						(
							setPropertyController outputCam #whiteBalance (getPropertyController inputCam #white_balance_custom)
						)
						catch 
						(
							setProperty outputCam #whiteBalance ((getProperty inputCam #white_balance_custom)*colorCorr)
						)					
					)
				)		
				--Bokeh
				case inputCam.bokeh_shape of
				(
					0:	--circular
					(
						outputCam.use_Blades = false
					)
					1:	--bladed
					(
						outputCam.use_Blades = true
						try 
						(
							setPropertyController outputCam #blades_number (getPropertyController inputCam #bokeh_blades_number )
						)
						catch 
						(
							setProperty outputCam #blades_number (getProperty inputCam #bokeh_blades_number )
						)
						try 
						(
							setPropertyController outputCam #blades_rotation (getPropertyController inputCam #pb_bokeh_blades_rotation_degrees )
						)
						catch 
						(
							setProperty outputCam #blades_rotation (getProperty inputCam #pb_bokeh_blades_rotation_degrees )
						)
					)
					2:	--textured
					(
						outputCam.bitmap_aperture_on = true
						outputCam.bitmap_aperture_affectExposure = inputCam.bokeh_texture_affect_exposure
						outputCam.bitmap_aperture_texmap = inputCam.bokeh_texture
						outputCam.bitmap_aperture_res = inputCam.baseObject.ReferenceTarget.bitmap_aperture_res
					)
				)
				try 
				(
					setPropertyController outputCam #center_bias (getPropertyController inputCam #bokeh_center_bias )
				)
				catch 
				(
					setProperty outputCam #center_bias (getProperty inputCam #bokeh_center_bias )
				)			
				try 
				(
					setPropertyController outputCam #optical_vignetting (getPropertyController inputCam #bokeh_optical_vignetting )
				)
				catch 
				(
					setProperty outputCam #optical_vignetting (getProperty inputCam #bokeh_optical_vignetting )
				)			
				try 
				(
					setPropertyController outputCam #anisotropy (getPropertyController inputCam #bokeh_anisotropy )
				)
				catch 
				(
					setProperty outputCam #anisotropy (getProperty inputCam #bokeh_anisotropy )
				)			
				--Distortions
				case inputCam.distortion_type of 
				(
					0:	--none
					(
						outputCam.distortion_type = 0
						outputCam.distortion = 0
					)
					1:	--cubic
					(
						outputCam.distortion_type = 1
						try 
						(
							setPropertyController outputCam #distortion (getPropertyController inputCam #distortion_cubic_amount )
						)
						catch 
						(
							setProperty outputCam #distortion (getProperty inputCam #distortion_cubic_amount )
						)
					)
					2:	--textured
					(
						outputCam.distortion_type = 3
						outputCam.distortion_map = inputCam.distortion_texture
					)
				)
				--lens file
				if inputCam.baseObject.ReferenceTarget.use_lens_file do 
				(
					outputCam.distortion_type = 2
					outputCam.lens_file = inputCam.baseObject.ReferenceTarget.lens_file
				)
			)
			
			--transforms
			outputCam.position.controller = copy inputCam.position.controller
			outputCam.scale.controller = copy inputCam.scale.controller				
	
			if inputCam.target != undefined then 
			(
				outputCam.transform.controller.Roll_Angle.controller = copy inputCam.transform.controller.Roll_Angle.controller
				outputCam.target.position.controller = copy inputcam.target.position.controller
			)
			else
			(
				outputCam.rotation.controller = copy inputCam.rotation.controller	
			)

		)
	)

	theCameras = for c in (selection as array) where classOf c == physical collect c
	if theCameras.count == 0 do 
	(
		theCameras = for c in cameras where classOf c == physical collect c
	)
	for c in theCameras do copyCameraControllers c 

	/*
		These are the only parameters left over without a correspondence.
		They are always calculated by the max physical, there is no property named like this.
	  .shutter_angle : float
	  .shutter_offset : float

	*/

)

